--rendering order example; without metatable
--love 0.8.0
function love.load()
  --settings
  math.randomseed(os.time())
  love.graphics.setDefaultImageFilter("nearest", "nearest")

  --MACRO
  SCALE = 2 --DRAWING_SCALE
  DEBUG = true
  GROUND_X = 15
  GROUND_Y = 100
  local SPRITE_GRASS_COUNT = 10
  
  --resource table
  tRes = {
    player = love.graphics.newImage("thing.png");
    grass = love.graphics.newImage("grass.png");
  }
  for i = 1, 3 do--generate random grass spriteBatch; store in the resource table tRes
    local spriteBatch = love.graphics.newSpriteBatch(tRes.grass, 10, "static")
    for i = 1, SPRITE_GRASS_COUNT do
      spriteBatch:add(math.random(0, SPRITE_GRASS_COUNT*5*SCALE), 0)
    end
    tRes["grassSpriteBatch" .. i] = spriteBatch
  end
  
  --define objects
  player = {name = "player"; x = 0; y = 0; image = tRes.player; imageH = tRes.player:getHeight()}
    function player:update(dt, mx, my)
      self.x = (mx)/SCALE - GROUND_X--move player x position according to mouse x position
    end
    function player:draw(groundX, groundY)
      love.graphics.draw(self.image, (groundX + self.x)*SCALE, (groundY - self.y - self.imageH*2)*SCALE, 0, SCALE*2)
    end
  
  grass1 = {name = "grass1"; x = 0; y = 0; spriteBatch = tRes.grassSpriteBatch1; imageH = tRes.grass:getHeight();}
  grass2 = {name = "grass2"; x = 50; y = 0; spriteBatch = tRes.grassSpriteBatch2; imageH = tRes.grass:getHeight();}
  grass3 = {name = "grass3"; x = 100; y = 0; spriteBatch = tRes.grassSpriteBatch3; imageH = tRes.grass:getHeight();}
    function grass1:draw(groundX, groundY)
      love.graphics.setColor(255, 200, 200, 255)
      love.graphics.draw(self.spriteBatch, (groundX + self.x)*SCALE, (groundY - self.y - self.imageH)*SCALE, 0, SCALE)
      love.graphics.setColor(255, 255, 255, 255)
    end
    function grass2:draw(groundX, groundY)
      love.graphics.draw(self.spriteBatch, (groundX + self.x)*SCALE, (groundY - self.y - self.imageH)*SCALE, 0, SCALE)
    end
    function grass3:draw(groundX, groundY)
      love.graphics.setColor(255, 150, 255, 255)
      love.graphics.draw(self.spriteBatch, (groundX + self.x)*SCALE, (groundY - self.y - self.imageH)*SCALE, 0, SCALE)
      love.graphics.setColor(255, 255, 255, 255)
    end
  
  --object manager
  tObject = {}
  function tObject:register(object, renderingOrder)--add object to the object manager
    if renderingOrder and renderingOrder <= #self then
      table.insert(self, renderingOrder, object)
    else
      table.insert(self, object)
    end
    if DEBUG then self:debug() end
  end
  function tObject:update(dt, mx, my)
    for i, object in ipairs(self) do
      if object.update then
        object:update(dt, mx, my)
      end
    end
  end
  function tObject:draw(groundX, groundY)
    for i, object in ipairs(self) do
      object:draw(groundX, groundY)
      if DEBUG then love.graphics.print(object.name, (groundX + object.x)*SCALE, (groundY)*SCALE) end
    end
  end
  function tObject:debug()
    for i, object in ipairs(self) do
      print(i, object.name)
    end
    print()
  end
  
  
  --setup objects in the object manager
  tObject:register(player)
  tObject:register(grass1, 1)--put grass1 at the back
  tObject:register(grass2)
  tObject:register(grass3, 1)--put grass1 at the back
end

function love.update(dt)
  tObject:update(dt, love.mouse.getPosition())
end

function love.draw()
  tObject:draw(GROUND_X, GROUND_Y)
  love.graphics.line(GROUND_X*SCALE, GROUND_Y*SCALE, love.graphics.getWidth() - GROUND_X*SCALE, GROUND_Y*SCALE)
end

function love.keypressed(key)
  if key == "f5" then
    love.filesystem.load("main.lua")()
    love.load()
  elseif key == "escape" then
    love.event.push("quit")
  end
end
